﻿namespace UltraGridPrintWithLegend
{
    using System;
    using System.Collections.Generic;
    using System.Data;
    using System.Drawing;
    using System.Windows.Forms;
    using Infragistics.Win.Printing;

    public partial class Form1 : Form
    {
        private int rows;
        private int cols;
        private ICollection<LegendItem> legendItems;
        private Random random;
        private Size maxItemSize;

        public Form1()
        {
            this.InitializeComponent();
            this.rows = 100;
            this.cols = 10;
            this.legendItems = new HashSet<LegendItem>();
            this.random = new Random();
            this.maxItemSize = Size.Empty;
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            this.ultraGrid1.SetDataBinding(this.GetData(this.rows, this.cols), null);
            this.ultraGridPrintDocument1.FitWidthToPages = 1;
        }

        private object GetData(int rows, int cols)
        {
            DataTable dt = new DataTable();

            for (int col = 0; col < cols; col++)
            {
                dt.Columns.Add(string.Format("Column #{0}", col + 1));
            }

            for (int row = 0; row < rows; row++)
            {
                DataRow addRow = dt.NewRow();
                foreach (DataColumn column in dt.Columns)
                {
                    addRow[column] = string.Format("{0}.{1}", row + 1, column.Ordinal + 1);
                }

                dt.Rows.Add(addRow);
            }

            return dt;
        }

        private ICollection<LegendItem> GetRandomCountOfLegendItems()
        {
            int count = this.random.Next(5, 10);
            HashSet<LegendItem> items = new HashSet<LegendItem>();
            for (int item = 0; item < count; item++)
            {
                string description = string.Format("This is legend item #{0}", item + 1);
                Color foreColor = this.GetRandomColor();
                Color backgroundColor = this.GetRandomColor();
                LegendItem newItem = new LegendItem()
                {
                    Description = description,
                    ForeColor = foreColor,
                    Background = backgroundColor,
                };

                items.Add(newItem);
            }

            return items;
        }

        private Color GetRandomColor()
        {
            int r = this.random.Next(0, 256);
            int g = this.random.Next(0, 256);
            int b = this.random.Next(0, 256);
            return Color.FromArgb(r, g, b);
        }

        private Size GetLegendHeight(ICollection<LegendItem> legendItems)
        {
            SizeF maxItemSize = SizeF.Empty;
            using (Graphics g = this.ultraGrid1.CreateGraphics())
            {
                foreach (LegendItem item in legendItems)
                {
                    SizeF currentItemSize = g.MeasureString(item.Description, this.Font);
                    if (maxItemSize.Height < currentItemSize.Height)
                    {
                        maxItemSize.Height = currentItemSize.Height;
                    }

                    if (maxItemSize.Width < currentItemSize.Width)
                    {
                        maxItemSize.Width = currentItemSize.Width;
                    }
                }
            }

            // adding some padding to the items
            maxItemSize.Height += 4;

            return maxItemSize.ToSize();
        }

        private void ButtonPrint_Click(object sender, EventArgs e)
        {
            this.ultraGridPrintDocument1.Grid = this.ultraGrid1;
            this.legendItems = this.GetRandomCountOfLegendItems();
            this.maxItemSize = this.GetLegendHeight(this.legendItems);
            int pageBodyWidth = this.GetPageWidth();
            this.ultraGridPrintDocument1.PageBody.Padding.Bottom = maxItemSize.Height * this.legendItems.Count + 4;
            this.ultraGridPrintDocument1.Print();
        }

        private int GetPageWidth()
        {
            int entireWidth = this.ultraGridPrintDocument1.DefaultPageSettings.Bounds.Width;
            int leftMargin = this.ultraGridPrintDocument1.DefaultPageSettings.Margins.Left;
            int rightMargin = this.ultraGridPrintDocument1.DefaultPageSettings.Margins.Right;
            return entireWidth - leftMargin - rightMargin;
        }

        private void UltraGridPrintDocument1_PageBodyPrinting(object sender, PageBodyPrintingEventArgs e)
        {
            Point recLocation = new Point(
                e.RectInsideBorders.Left,
                e.RectInsidePadding.Bottom + 1);

            Size recSize = new Size(
                e.RectInsideBorders.Right - recLocation.X - 1,
                e.RectInsideBorders.Bottom - recLocation.Y - 1);

            Rectangle legendRec = new Rectangle(recLocation, recSize);
            e.Graphics.DrawRectangle(Pens.Red, legendRec);

            Point startLocation = recLocation;
            startLocation.X += 1;
            startLocation.Y += 1;
            foreach (LegendItem item in this.legendItems)
            {
                Brush b = new SolidBrush(item.Background);
                e.Graphics.FillRectangle(b, new Rectangle(startLocation, this.maxItemSize));

                b = new SolidBrush(item.ForeColor);
                e.Graphics.DrawString(item.Description, this.Font, b, startLocation.X, startLocation.Y + 2);
                b.Dispose();
                startLocation.Y += this.maxItemSize.Height;
            }
        }
    }
}
